#include "efm32.h"

#define LETIMERBASE	1	// 16KHz
#define TIMERBASE	874	// 32KHz
#define SYSTICKBASE	27999	// 1ms
#define ADCWUBASE	27	// /28
#define ADCPRSCLR	2	// /3 (9MHz)
#define DACPRSCLR	7	// /128 (187.5KHz)


#define SETUPPINs GPIO->P[2].MODEL &= ~(_GPIO_P_MODEL_MODE1_MASK+_GPIO_P_MODEL_MODE2_MASK+_GPIO_P_MODEL_MODE0_MASK); \
		GPIO->P[2].MODEL |= GPIO_P_MODEL_MODE1_PUSHPULL+GPIO_P_MODEL_MODE2_PUSHPULL+GPIO_P_MODEL_MODE0_PUSHPULL;
#define SETPIN1 GPIO->P[2].DOUT |= (1<<1);
#define RESETPIN1 GPIO->P[2].DOUT &= ~(1<<1);
#define SETPIN2 GPIO->P[2].DOUT |= (1<<2);
#define RESETPIN2 GPIO->P[2].DOUT &= ~(1<<2);
#define SETPIN3 GPIO->P[2].DOUT |= (1<<0);
#define RESETPIN3 GPIO->P[2].DOUT &= ~(1<<0);
#define SETPINs GPIO->P[2].DOUT |= (1<<1)+(1<<2)+(1<<0);


volatile unsigned long SysTickCnt;      // SysTick Counter

// SysTick Setup
void SysTick_Setup (void)
{
  SysTick->LOAD = SYSTICKBASE;
  SysTick->CTRL = 0x0007;               // Enable Counter and Tick Interrupt
}

void SysTick_Handler (void)
{
  SysTickCnt++;
}

void init_EFM32(void);
void init_timer(void);
void init_ADC(void);
void init_DAC(void);



void main()
{

	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_GPIO;		// enable clock
	GPIO->P[1].MODEH =
		GPIO_P_MODEH_MODE11_DISABLED;			// PB11 for DAC (GPIO disabled)
	GPIO->P[3].MODEL =
		GPIO_P_MODEL_MODE4_DISABLED;			// PD4 for ADC (GPIO disabled)
	SETUPPINs;
	init_EFM32();
	init_ADC();
	init_DAC();
	init_timer();
//	SysTick_Setup();
	while(1){
		__WFI();
	};
}


void init_EFM32(void)
{
uint32_t tuning;

	CMU->OSCENCMD = CMU_OSCENCMD_HFRCOEN;
	MSC->READCTRL = 1;							// need 1WS above 16MHz
	// HFRCO
	tuning = (DEVINFO->HFRCOCAL1 & _DEVINFO_HFRCOCAL1_BAND21_MASK) >>	// calib for 28MHz
		_DEVINFO_HFRCOCAL1_BAND21_SHIFT;
	CMU->HFRCOCTRL = (CMU->HFRCOCTRL &
		~(_CMU_HFRCOCTRL_BAND_MASK | _CMU_HFRCOCTRL_TUNING_MASK)) |
		(5 << _CMU_HFRCOCTRL_BAND_SHIFT) |				// 28MHz band
		(tuning << _CMU_HFRCOCTRL_TUNING_SHIFT);
	// core clock
	CMU->HFCORECLKDIV = CMU_HFCORECLKDIV_HFCORECLKDIV_HFCLK;		// no divisor
	CMU->CMD = CMU_CMD_HFCLKSEL_HFRCO;					// select HFRCO for HFCLK
#ifndef USETIMER0
	/* sta se me pas en el libro... perdn.
	Cuando se usa un perifrico "low-energy" hay que habilitar el clock 
	en la interfaz entre los diferentes dominios de clock */
	CMU->HFCORECLKEN0 = CMU_HFCORECLKEN0_LE;				// enable Low Energy domain (low-frequency) interface
#endif
	// LFRCO
	CMU->OSCENCMD = CMU_OSCENCMD_LFRCOEN;
	CMU->LFCLKSEL =
		CMU_LFCLKSEL_LFB_LFRCO |					// select LFRCO for LFBCLK
		CMU_LFCLKSEL_LFA_LFRCO ;					// select LFRCO for LFACLK
}

void init_ADC()
{
uint32_t tmp;

	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_ADC0;				// enable clock
	/* Make sure conversion is not in progress */
	ADC0->CMD = ADC_CMD_SINGLESTOP | ADC_CMD_SCANSTOP;

	ADC0->CTRL =
		((ADCWUBASE << _ADC_CTRL_TIMEBASE_SHIFT) & _ADC_CTRL_TIMEBASE_MASK) | // 1us time base for warmup
		((ADCPRSCLR << _ADC_CTRL_PRESC_SHIFT) & _ADC_CTRL_PRESC_MASK) | // prescaler divisor
		(0 << _ADC_CTRL_LPFMODE_SHIFT) |				// no LPF
		(0 << _ADC_CTRL_WARMUPMODE_SHIFT);				// normal warmup

	/* Load proper calibration data */
	tmp  = ADC0->CAL & ~(_ADC_CAL_SINGLEOFFSET_MASK | _ADC_CAL_SINGLEGAIN_MASK);
	tmp |= ((DEVINFO->ADC0CAL0 & _DEVINFO_ADC0CAL0_2V5_GAIN_MASK) >>
		_DEVINFO_ADC0CAL0_2V5_GAIN_SHIFT) << _ADC_CAL_SINGLEGAIN_SHIFT;
	tmp |= ((DEVINFO->ADC0CAL0 & _DEVINFO_ADC0CAL0_2V5_OFFSET_MASK) >>
		_DEVINFO_ADC0CAL0_2V5_OFFSET_SHIFT) << _ADC_CAL_SINGLEOFFSET_SHIFT;
	ADC0->CAL = tmp;

	ADC0->SINGLECTRL =
		(0 << _ADC_SINGLECTRL_AT_SHIFT) |				// 1 cycle acquisition time
		(1 << _ADC_SINGLECTRL_REF_SHIFT) |				// 2.5V reference
	        (4 << _ADC_SINGLECTRL_INPUTSEL_SHIFT) |				// input is ch4 (PD4)
		(0 << _ADC_SINGLECTRL_RES_SHIFT);				// 12-bit resolution
	ADC0->IEN = ADC_IEN_SINGLE;						// enable EOC interrupt
	NVIC_EnableIRQ(ADC0_IRQn);		// Enable ADC Interrupts
}

#include "efm32_bitband.h"

void init_DAC()
{
	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_DAC0;				// enable clock
	/* Make sure both channels are disabled. */
	BITBAND_Peripheral(&(DAC0->CH0CTRL), _DAC_CH0CTRL_EN_SHIFT, 0);
	BITBAND_Peripheral(&(DAC0->CH1CTRL), _DAC_CH1CTRL_EN_SHIFT, 0);

	/* Load proper calibration data depending on selected reference */
	DAC0->CAL = DEVINFO->DAC0CAL1;

	DAC0->CTRL =
		((DACPRSCLR << _DAC_CTRL_PRESC_SHIFT) & _DAC_CTRL_PRESC_MASK) |	// prescaler divisor
		DAC_CTRL_REFSEL_2V5 |						// 2.5V reference
		DAC_CTRL_CH0PRESCRST |						// reset prescaler on trigger
        	DAC_CTRL_OUTMODE_PIN |						// output to pin
		DAC_CTRL_CONVMODE_SAMPLEHOLD;					// sample and hold conversion
	DAC0->CH0CTRL =
		DAC_CH0CTRL_EN ;						// enable DAC
	DAC0->IEN = DAC_IEN_CH0;						// enable EOC interrupt
	NVIC_EnableIRQ(DAC0_IRQn);						// Enable Interrupts
}

void init_timer()
{
#ifdef USETIMER0
	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_TIMER0;				// enable clock
	TIMER0->CTRL =
		TIMER_CTRL_PRESC_DIV1 |						// no prescaler
		TIMER_CTRL_CLKSEL_PRESCHFPERCLK |				// use HFCLOCK
		TIMER_CTRL_FALLA_NONE |						// no special actions
		TIMER_CTRL_RISEA_NONE |						// no special actions
		TIMER_CTRL_MODE_DOWN ;						// down count
	TIMER0->TOP = TIMERBASE;						// divide by
	TIMER0->CMD = TIMER_CMD_START;
	TIMER0->IEN = TIMER_IEN_UF;						// enable underflow interrupt
	NVIC_EnableIRQ(TIMER0_IRQn);						// Enable Interrupts
#else
/* Use LETIMER */
	CMU->LFAPRESC0 = CMU_LFAPRESC0_LETIMER0_DIV1;				// no prescaler for LETIMER0
	CMU->LFACLKEN0 |= CMU_LFACLKEN0_LETIMER0;				// enable clock
	LETIMER0->FREEZE = LETIMER_FREEZE_REGFREEZE_FREEZE;			// freeze (low frequency domain)
	LETIMER0->CTRL =
		LETIMER_CTRL_COMP0TOP |						// top count is COMP0
		LETIMER_CTRL_UFOA0_PULSE |					// pulse output LET0_O0 on underflow
		LETIMER_CTRL_REPMODE_FREE;					// free run		
	LETIMER0->COMP0 = LETIMERBASE;						// top (divide by top+1)
	LETIMER0->REP0 = 1;							// must be =! 0 so output 0 can switch (?)
	LETIMER0->CMD = LETIMER_CMD_START;
	LETIMER0->IEN = LETIMER_IEN_UF;						// enable underflow interrupt
	LETIMER0->FREEZE = LETIMER_FREEZE_REGFREEZE_UPDATE;			// "defrost" (low frequency domain)
	NVIC_EnableIRQ(LETIMER0_IRQn);						// Enable Interrupts
#endif
}


uint16_t datain,dataout;

#ifdef USETIMER0
void TIMER0_IRQHandler()
#else
void LETIMER0_IRQHandler()
#endif
{
	ADC0->CMD=ADC_CMD_SINGLESTART;						// start conversion
	DAC0->CH0DATA=dataout;
	SETPINs;								// see it @PIN
	dataout=datain;
	RESETPIN1;								// see it @PIN
	SCB->SCR &= ~SCB_SCR_SLEEPDEEP_Msk;					// prevent HFCLK disable
#ifdef USETIMER0
	TIMER0->IFC = TIMER_IFC_UF;						// ACK
#else
	LETIMER0->IFC = LETIMER_IFC_UF;						// ACK
#endif
}

void ADC0_IRQHandler()
{
//uint16_t data;

	datain=(int16_t)(ADC0->SINGLEDATA);
	ADC0->IFC = ADC_IFC_SINGLE;						// ACK
	RESETPIN2;								// see it @PIN
}

void DAC0_IRQHandler()
{
	DAC0->IFC = DAC_IFC_CH0;						// ACK
	RESETPIN3;								// see it @PIN
	SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;					// enable deep sleep
}

